#include "angle.h"
#include <wx/math.h>
#include <float.h>

Angle::Angle(double value, Unit unit)
{
	if( unit == Deg )
		radians = degreesToRadians(value);
	else
		radians = value;
}

double Angle::getDegrees() const
{
	return radiansToDegrees(radians);
}

void Angle::setDegrees(double degrees)
{
	radians = degreesToRadians(degrees);
}

Angle Angle::abs(const Angle &a)
{
	return Angle( fabs(a.radians) );
}

bool Angle::isNaN(Angle &a)
{
	return wxIsNaN(a.radians);
}

bool Angle::isNaN() const
{
	return wxIsNaN(radians);
}

bool Angle::operator==(const Angle &a) const
{
	return fabs(radians - a.radians) < DBL_EPSILON;
}

bool Angle::operator!=(const Angle &a) const
{
	return fabs(radians - a.radians) > DBL_EPSILON;
}

bool Angle::operator<(const Angle &a) const
{
	return radians < a.radians;
}

bool Angle::operator>(const Angle &a) const
{
	return radians > a.radians;
}

Angle Angle::operator+(const Angle &a) const
{
	double res = radians + a.radians;
	return Angle(res);
}

Angle& Angle::operator+=(const Angle &a)
{
	radians += a.radians;
	return *this;
}

Angle Angle::operator-(const Angle &a) const
{
	double res = radians - a.radians;
	return Angle(res);
}

Angle& Angle::operator-=(const Angle &a)
{
	radians -= a.radians;
	return *this;
}

Angle Angle::operator*(double times) const
{
	return Angle(radians * times);
}

Angle operator*(double times, Angle &a)
{
	return Angle(a.radians * times);
}

Angle operator/(double divisor, Angle &a) 
{
	return Angle(a.radians / divisor);
}

Angle Angle::operator/(double divisor) const
{
	return Angle(radians / divisor);
}

double Angle::degreesToRadians(double degrees)
{
	return degrees * M_PI / 180.;
}

double Angle::radiansToDegrees(double radians)
{
	return radians * 180. / M_PI;
}
