#include "downloadmanager.h"
#include "fileinformation.h"
#include "nawiframe.h"
#include <wx/curl/thread.h>
#include <wx/log.h>
#include <wx/wfstream.h>
#include <wx/listimpl.cpp>

WX_DEFINE_LIST(DownloadQueue);

BEGIN_EVENT_TABLE(DownloadManager, wxEvtHandler)
	EVT_CURL_END_PERFORM(ID_DOWNLOAD_THREAD, DownloadManager::onDownloadComplete)
END_EVENT_TABLE()

DownloadManager::DownloadManager()
	:
	wxEvtHandler(),
	currentDownload(NULL)
{
	downloadThread = new wxCurlDownloadThread(this, ID_DOWNLOAD_THREAD);
	downloadQueue = new DownloadQueue();
}

DownloadManager::~DownloadManager()
{
	delete downloadThread;
	delete downloadQueue;
}

void DownloadManager::add(FileInformation *fileInfo)
{
	downloadQueue->push_back(fileInfo);

	// Start download (if the queue was empty)
	startNextDownload();
}

void DownloadManager::startNextDownload()
{
	if( downloadQueue->size() > 0 && !downloadThread->IsAlive() )
	{
		currentDownload = downloadQueue->front();
		wxLogMessage("Downloading " + currentDownload->url + currentDownload->fileName + " to " + currentDownload->path);
		wxFileName::Mkdir( currentDownload->path, 0777, wxPATH_MKDIR_FULL );
		downloadQueue->pop_front();
		currentDownload->downloading = true;
		downloadThread->Download( currentDownload->url + currentDownload->fileName, new wxFileOutputStream(currentDownload->path + currentDownload->fileName + ".tmp") );
	}
}

void DownloadManager::onDownloadComplete( wxCurlEndPerformEvent &event )
{
	if( event.IsSuccessful() && currentDownload->localTmpFileExists() )
	{
		downloadThread->GetOutputStream()->Close();
		currentDownload->renameTmpFile();
	}
	else
		currentDownload->downloadFailed = true;
	currentDownload->downloading = false;
}
