#include "layer.h"
#include "exception.h"
#include "nawiglcanvas.h"
#include "camera.h"
#include <wx/filename.h>
#include <wx/log.h>

Layer::Layer(const wxString &name, Layer *parentLayer)
	:
	isInitialized(false),
	parentLayer(parentLayer),
	name(name),
	isOn(true),
	updatedSinceLastRender(false)
{
}

Layer::Layer(const wxString &name, const Vector &position, const Quaternion &orientation, Layer *parentLayer)
	:
	isInitialized(false),
	parentLayer(parentLayer),
	position(position),
	name(name),
	orientation(orientation),
	isOn(true),
	updatedSinceLastRender(false)
{
}

Layer::~Layer()
{
	for(int i = 0; i < children.GetCount(); i++)
		delete children[i];
}

void Layer::initialize()
{
	if (!isOn)
		return;

	for(int i = 0; i < children.GetCount(); i++)
	{
		try
		{
			if (children[i]->isOn)
				children[i]->initialize();
		}
		catch (Exception &e)
		{
			wxLogError(name + wxString(": ") + e.message + wxString(" (") + children[i]->name + ")");
		}
	}

	isInitialized = true;
}

void Layer::update()
{
	if (!isOn)
		return;

	if (!isInitialized)
		initialize();

	for(int i = 0; i < children.GetCount(); i++)
	{
		Layer *l = children[i];
		if( !l->parentLayer )
			l->parentLayer = this;

		if( l->isOn )
			l->update();
	}
}

void Layer::render(NawiGLCanvas *glCanvas)
{
	renderChildren(glCanvas);
}

void Layer::renderChildren(NawiGLCanvas *glCanvas)
{
	if( !isOn/* && updatedSinceLastRender*/ )
		return;

	try
	{
		for(int i = 0; i < children.GetCount(); i++)
		{
			Layer *l = children[i];
			if( l->isOn/* && l->updatedSinceLastRender*/ )
			{
				l->renderChildren(glCanvas);
				l->render(glCanvas);
			}
		}
	}
	catch (Exception &e)
	{
		e.log();
	}
}

void Layer::flagUpdatedLayers()
{
//	updatedSinceLastRender = true;
	if( parentLayer )
		parentLayer->flagUpdatedLayers();
	else
		updatedSinceLastRender = true;
}
